package io.tswf.minecraft.blockventure.core.gui

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.nbt.NBTObjectMapper
import io.tswf.minecraft.blockventure.core.api.network.packet.NetworkPlayPacketSender
import io.tswf.minecraft.blockventure.core.api.registry.GuiScreenRegistry
import io.tswf.minecraft.blockventure.core.api.thread.MinecraftThreadsExecutor
import io.tswf.minecraft.blockventure.core.exception.NoSuchGuiScreenProviderFoundException
import net.minecraft.client.MinecraftClient
import net.minecraft.client.gui.screen.Screen
import net.minecraft.entity.player.PlayerEntity
import net.minecraft.screen.PlayerScreenHandler
import net.minecraft.screen.ScreenHandler
import net.minecraft.server.network.ServerPlayerEntity
import net.minecraft.world.World

@CompileStatic
class PlayerScreenService {

    public static final int NO_SCREEN_HANDLER_WINDOW_ID = Integer.MIN_VALUE

    static int getNextScreenHandlerSyncId(ServerPlayerEntity player) {
        if (player.currentScreenHandler !instanceof PlayerScreenHandler) {
            player.closeHandledScreen()
        }

        player.incrementScreenHandlerSyncId()
        return player.screenHandlerSyncId
    }

    static void openScreen(String modId, PlayerEntity player, String screenId, int windowId, World world, int x, int y, int z, Object customPayload) {
        def guiScreenProvider = GuiScreenRegistry.registeredGuiScreenProviders[modId]
                .ensureNotNull { new NoSuchGuiScreenProviderFoundException("There is no gui screen provider found for mod id: ${modId}!") }

        def screenHandler = (ScreenHandler) guiScreenProvider.getScreenHandler(world, x, y, z, player, windowId, screenId, customPayload)

        if (screenHandler != null && windowId != NO_SCREEN_HANDLER_WINDOW_ID) {
            throw new IllegalStateException("Trying to open handled screen from client. It is only available from server!")
        }

        if (world.isClient) {
            MinecraftThreadsExecutor.runInMinecraftClientThread {
                def guiScreen = (Screen) guiScreenProvider.getGuiScreen(world, x, y, z, player, windowId, screenId, customPayload)
                MinecraftClient.getInstance().screen = guiScreen
                player.currentScreenHandler = screenHandler
            }
        } else {
            MinecraftThreadsExecutor.runInMinecraftServerThread {
                def serverPlayer = (ServerPlayerEntity) player
                player.currentScreenHandler = screenHandler

                if (screenHandler != null) {
                    serverPlayer.onScreenHandlerOpened(screenHandler)
                    serverPlayer.screenHandlerSyncId = windowId
                } else if (serverPlayer.currentScreenHandler !instanceof PlayerScreenHandler) {
                    serverPlayer.closeHandledScreen()
                }

                def openAtGuiPacket = new OpenScreenAtClientPacket(
                        xCoord: x,
                        yCoord: y,
                        zCoord: z,
                        windowId: windowId,
                        guiScreenId: screenId,
                        modId: modId
                )

                if (customPayload != null) {
                    def serializedCustomPayload = NBTObjectMapper.mapToNbt(customPayload, customPayload.class)
                    if (serializedCustomPayload == null) {
                        throw new IllegalStateException("Custom payload '${customPayload}' with class '${customPayload?.class?.name}' was not serialized!")
                    }

                    openAtGuiPacket.customData = serializedCustomPayload
                    openAtGuiPacket.customDataClass = customPayload.class.name
                }

                NetworkPlayPacketSender.sendPacketToPlayerClient(openAtGuiPacket, serverPlayer)
            }
        }
    }
}
