package io.tswf.minecraft.blockventure.core.inventory.generic

import groovy.transform.CompileStatic
import io.tswf.minecraft.blockventure.core.api.annotation.MapNBT
import io.tswf.minecraft.blockventure.core.api.nbt.IReflectiveNBTObject
import io.tswf.minecraft.blockventure.core.api.nbt.NBTObjectMapper
import net.minecraft.entity.player.PlayerEntity
import net.minecraft.inventory.Inventories
import net.minecraft.inventory.Inventory
import net.minecraft.item.ItemStack
import net.minecraft.nbt.NbtCompound
import net.minecraft.nbt.NbtElement
import net.minecraft.nbt.NbtTypes
import net.minecraft.util.collection.DefaultedList

import java.util.function.Consumer
import java.util.function.Predicate

@CompileStatic
class GenericInventoryHolder implements Inventory {

    public static String defaultInventoryName = System.findGlobalProperty("blockventure.core.inventory.generic.default-name")
        .orElse("Inventory")

    String inventoryName
    GenericInventoryData inventoryData
    Predicate<PlayerEntity> canPlayerUseInventoryPredicate = { true }

    List<Consumer<GenericInventoryHolder>> inventoryUpdateListeners = new ArrayList<>()

    GenericInventoryHolder(int inventorySize, String inventoryName = defaultInventoryName) {
        inventoryData = new GenericInventoryData(
                stacks: DefaultedList.ofSize(inventorySize, ItemStack.EMPTY)
        )

        this.inventoryName = inventoryName
    }

    @Override
    int size() {
        return inventoryData.stacks.size()
    }

    @Override
    boolean isEmpty() {
        return inventoryData.stacks.stream().allMatch { it.isEmpty() }
    }

    @Override
    ItemStack getStack(int slot) {
        return slot >= 0 && slot < this.inventoryData.stacks.size() ? (ItemStack)this.inventoryData.stacks.get(slot) : ItemStack.EMPTY
    }

    @Override
    ItemStack removeStack(int slot, int amount) {
        return Inventories.splitStack(this.inventoryData.stacks, slot, amount)
    }

    @Override
    ItemStack removeStack(int slot) {
        return Inventories.removeStack(this.inventoryData.stacks, slot)
    }

    void setStack(int slot, ItemStack stack) {
        if (slot >= 0 && slot < this.inventoryData.stacks.size()) {
            this.inventoryData.stacks.set(slot, stack)
        }
    }

    @Override
    void markDirty() {
        inventoryUpdateListeners?.with {
            forEach { listener ->
                listener.accept(this)
            }
        }
    }

    @Override
    boolean canPlayerUse(PlayerEntity player) {
        return canPlayerUseInventoryPredicate?.test(player) ?: false
    }

    @Override
    void clear() {
        inventoryData.stacks.clear()
    }

    void writeToNbt(NbtCompound compound) {
        def inventoryDataCompound = NBTObjectMapper.mapToNbt(inventoryData, GenericInventoryData)
        compound.put(inventoryName, inventoryDataCompound)
    }

    void readFromNbt(NbtCompound compound) {
        if (compound.contains(inventoryName, NbtElement.COMPOUND_TYPE)) {
            def inventoryDataCompound = compound.getCompound(inventoryName)
            def inventoryData = NBTObjectMapper.mapToObject(inventoryDataCompound, GenericInventoryData, null)

            if (inventoryData instanceof GenericInventoryData) {
                this.inventoryData = inventoryData
            }
        }
    }

    @CompileStatic
    static class GenericInventoryData implements IReflectiveNBTObject {

        @MapNBT
        DefaultedList<ItemStack> stacks
    }
}
